"""
enscons' implementation of setup.py, to be called from a shim setup.py for
compatibility with traditional style packaging & tools.
"""

import sys, pkg_resources, argparse


def develop(path):
    """
    Add distribution on path to easy-install.pth for development, like setup.py develop.
    A single egg-info should exist in path.
    This may work fine with a .dist-info directory in place of .egg-info.
    """
    import os
    from setuptools.command import easy_install
    from . import paths

    pathdir = paths.get_install_paths("enscons")["purelib"]
    pathfile = os.path.join(pathdir, "easy-install.pth")

    pthdistributions = easy_install.PthDistributions(pathfile)
    # distribution = Distribution(
    #     target,
    #     PathMetadata(target, os.path.abspath(ei.egg_info)),
    #     project_name=ei.egg_name)
    distribution = list(pkg_resources.find_distributions(path, True))[0]
    pthdistributions.add(distribution)
    pthdistributions.save()

    egg_link = os.path.join(pathdir, distribution.project_name + ".egg-link")
    egg_path = os.path.abspath(path)
    with open(egg_link, "w") as f:
        f.write(egg_path + "\n" + ".")
        f.close()


def setup():
    """
    Adjust some command line arguments for SCons, then run.
    To be called at the bottom of a build-dependency-shim setup.py.
    """

    parser = argparse.ArgumentParser(description="setup.py arguments")
    parser.add_argument("--egg-base", action="store", dest="base")
    parser.add_argument("--python-tag", action="store", dest="python_tag")
    parser.add_argument("--formats", type=str, action="store", dest="sdist_formats")
    parser.add_argument("--dist-dir", type=str, action="store", dest="sdist_dir")
    parser.add_argument("-d", action="store", dest="destination")
    parser.add_argument("--no-deps", action="store_true", dest="no_deps")
    parser.add_argument("--single-version-externally-managed", action="store_true")
    parser.add_argument("--compile", action="store_true")
    parser.add_argument("--install-headers", action="store")
    parser.add_argument("--record", action="store")
    args, unknown = parser.parse_known_args(sys.argv)

    sys.argv = unknown  # pass along to SCons

    if len(sys.argv) > 1 and sys.argv[1] == "clean":  # pip calls this
        sys.exit(0)

    # sys.argv[0] is -c not 'setup.py' with pip

    # Convert certain arguments into =-separated format
    for flag, arg in (
        ("--dist-dir", "sdist_dir"),
        ("--wheel-dir", "destination"),
        ("--egg-base", "base"),
    ):
        if getattr(args, arg):
            sys.argv.append("=".join((flag, getattr(args, arg))))

    if "develop" in sys.argv:
        src_root = "."
        import pytoml

        with open("pyproject.toml", "r") as pyproject:
            metadata = pytoml.load(pyproject)
            src_root = metadata["tool"]["enscons"].get("src_root", src_root)
        develop(src_root)
        sys.argv.remove("develop")

    sys.path[0:0] = ["setup-requires"]
    pkg_resources.working_set.add_entry("setup-requires")

    import SCons.Script

    SCons.Script.main()


# Command used when installing from a source directory:
"""
"python3.5 -u -c "import setuptools, tokenize
__file__='/tmp/pip-6s1w5sd2-build/setup.py'
exec(compile(getattr(tokenize, 'open', open)(__file__).read().replace('\r\n', '\n'), __file__, 'exec'))"
install --record /tmp/pip-795f8wqp-record/install-record.txt
--single-version-externally-managed
--compile
--install-headers /home/dholth/prog/cryptacular/.tox/py35/include/site/python3.5/enscons
"""



#!/usr/bin/env python
# wgc "wheel greater compression"
# puts everything but *.dist-info/ in an interior archive

import pathlib
import os.path
import sys
import hashlib
import zipfile

from enscons import urlsafe_b64encode

compression=zipfile.ZIP_LZMA
compresslevel=None  # default

# top 512 wheels:
# original: 728M
# with default compresslevel: 678M
# with -9 compresslevel: 674M

# (savings in bytes) * (downloadcount) for last 30 days...

def add_manifest(wheel, dist_info):
    """
    Add the wheel manifest.
    """
    import hashlib
    import zipfile

    archive = zipfile.ZipFile(wheel, "a", compression=compression)
    lines = []
    for f in archive.namelist():
        data = archive.read(f)
        size = len(data)
        digest = hashlib.sha256(data).digest()
        digest = "sha256=" + (urlsafe_b64encode(digest).decode("ascii"))
        lines.append("%s,%s,%s" % (f.replace(",", ",,"), digest, size))

    record_path = dist_info + "/RECORD"
    lines.append(record_path + ",,")
    RECORD = "\n".join(lines)
    archive.writestr(record_path, RECORD, compresslevel=compresslevel)
    archive.close()

def greater(wheel):
    original = pathlib.Path(wheel)
    extra_crispy = original.with_suffix('.wgc')
    prefix = '-'.join(str(original.name).split('-', 2)[:2])
    dist_info = prefix + '.dist-info'
    data_zip = prefix + '.data.zip'

    for path in (extra_crispy, data_zip):
        p = pathlib.Path(path)
        if p.exists():
            p.unlink()

    with zipfile.ZipFile(original, 'r') as wheel_in:
        with zipfile.ZipFile(extra_crispy, 'w', compression=compression) as wheel_out:
            with zipfile.ZipFile(data_zip, 'w', compression=zipfile.ZIP_STORED) as interior:
                for info in wheel_in.filelist:
                    if not info.filename.startswith(dist_info):
                        interior.writestr(info.filename, wheel_in.read(info), compresslevel=compresslevel)
                    elif not info.filename.endswith('/RECORD'):
                        wheel_out.writestr(info.filename, wheel_in.read(info), compresslevel=compresslevel)
            wheel_out.write(data_zip, compresslevel=compresslevel)

    add_manifest(extra_crispy, dist_info)

    pathlib.Path(data_zip).unlink()

    return extra_crispy

for filename in sys.argv[1:]:
    try:
        output = greater(filename)
        sz_before = pathlib.Path(filename).stat().st_size
        sz_after = pathlib.Path(output).stat().st_size
        print("%s %0.0f%% %d" % (output, (sz_after / sz_before) * 100,  sz_after-sz_before))
    except Exception as e:
        print(e)
        print("%s fail" % filename)



#!/usr/bin/env python
# Call enscons to emulate setup.py, installing if necessary.
# (this setup.py can be copied into any enscons-powered project by editing requires=)

import sys, subprocess, os.path

sys.path[0:0] = ["setup-requires"]

try:
    import enscons.setup
except ImportError:
    requires = ["scons>=3.0.5", "pytoml", "wheel"]  # just ["enscons"] for enscons users
    subprocess.check_call(
        [sys.executable, "-m", "pip", "install", "-t", "setup-requires"] + requires
    )
    del sys.path_importer_cache["setup-requires"]  # needed if setup-requires was absent
    import enscons.setup

enscons.setup.setup()
